define([
    "dojo/dom-construct",
    "dijit/Dialog",
	"dijit/form/Button",
	"dojo/date/locale",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/serverRequest",
	"dojo/dom",
	"dojox/calendar/time",
	"dojo/currency",
	"gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "dojo/store/Memory"
],function(
    domConstruct,
    Dialog,
	Button,
	locale,
	lang,
	arrayUtils,
	serverRequest,
	dom,
	timeUtils,
	currency,
	Grid,
    Cache,
    SingleSort,
    Memory
) {
	var DATE_PATTERN = "yyyy-MM-dd";
	var DISPLAY_DATE_PATTERN = "MM/dd/yyyy";

	var formatters = {
		date: function(value) {
			var ret = "";
			if (value) {
				var date = locale.parse(value,{selector:"date",datePattern:DATE_PATTERN});
				ret = locale.format(date,{selector:"date",datePattern:DISPLAY_DATE_PATTERN});
			}
			return ret;
		},
		tableItem: function(value,table,labelGetter /*function(item) {return label} or attribute name*/ ) {
			var ret = "";
			labelGetter = typeof labelGetter !== "undefined" ? labelGetter : "name";
			if (table && table.get(value)) {
				var tableValue = table.get(value);
				if (lang.isFunction(labelGetter)) {
					ret = labelGetter(tableValue);
				} else {
					ret = tableValue[labelGetter];
				}
			}
			return ret;
		},
		def: function(value) {
			return value || "";
		},
		dollars: function(value) {
			var result = "$ - ";
			if (value !== "") {
				result = currency.format(value,{currency:"USD"});
			}
			return result;
		},
		percent: function(value) {
			return (value !== "" ? value+"%" : "");
		},
		boolean: function(value) {
			if (value == "") {
				return "";
			} else {
				return (value ? "Yes" : "No");
			}
		}
	};

	var createFormatter = function(type) {
		var formatter = formatters[type] || formatters.def;
		return function(column,first,second) {
			return function(item) {
				return formatter(item[column],first,second);
			};
		};
	};

	var showMessage = function(message,actions,closable,width) {
		message = message.toString().replace(/\n/g,"<br>");
		if (typeof closable === "undefined") {
			closable = true;
		}
		width = width || "300px";
		actions = actions || [{label:"OK"}];
		var messageDialog = new Dialog({
			title: "Message",
			content: "<span id='messageDialogContentSpan'>"+message+"</span>",
			style: "width: "+width,
			closable: closable,
			onCancel: function() {
				this.destroyRecursive();
			}
		});
		var actionBar = domConstruct.toDom("<div class='dijitDialogPaneActionBar'></div>");
		domConstruct.place(actionBar,messageDialog.containerNode);
		
		for (var i = 0; i < actions.length; i++) {
			var button = new Button({
				label: actions[i].label,
				onClick: function() {
					if (this.action) {
						this.action(this.args);
					}
					this.messageDialog.hide();
					this.messageDialog.destroyRecursive();
				},
				messageDialog: messageDialog
			}).placeAt(actionBar);
			button.action = actions[i].callback;
			button.args = actions[i].args;
		}
		messageDialog.show();
	};

	var createGrid = function(structure,node,allowSort,autoHeight) {
		autoHeight = (typeof autoHeight === "undefined" ? true : !!autoHeight);
		var modules = [];
		var modelExtensions = [];
		if (!!allowSort) {
			modules.push(SingleSort);
			modelExtensions.push("gridx/core/model/extensions/FormatSort");
		}
		var grid = new Grid({
			cacheClass: Cache,
			store: new Memory(),
			structure: structure,
			autoHeight: autoHeight,
			modules: modules,
			modelExtensions: modelExtensions,
			style: (autoHeight ? "" : "height:100%")
		});
		node.appendChild(grid.domNode);
		grid.startup();
		return grid;
	};

	var genGetEnvSetting = function(settingName) {
		return function() {
			try {
				if (settings && typeof settings[settingName] !== "undefined") {
					return settings[settingName];
				}
			} catch (e) {}
			return null;
		};
	};

	var fetchReportData = function(reportClass,ids,columns,async) {
		var context = {
			userId: genGetEnvSetting("userId")(),
			deviceAppId: genGetEnvSetting("deviceAppId")(),
			presentation: "Interactive",
			reportClass: reportClass,
			filters: [
				{
					column: "id",
					value: ids
				}
			],
			columns: columns,
			audit: false,
			filterFlag: true,
			pageStart: 0,
			pageSize: ids.length
		};
		return serverRequest(reportClass,"GenerateData",context,async);
	};	

	var checkPermission = function(permissionId,async) {
		var context = {
			permissionId: permissionId
		};
		return serverRequest("UI.Utility","CheckUserAccess",context,async);
	};	

    return {
		showMessage: showMessage,
		createGrid: createGrid,
		fetchReportData: fetchReportData,
		checkPermission: checkPermission,
		promptDownload: function(fileLinkId) {
			dom.byId("fileDownloadFrame").src = fileLinkId+"&prompt=true";
		},
		environment: {
			getIsServer: 	genGetEnvSetting("isServer"),
			getUserId: 		genGetEnvSetting("userId"),
			getDeviceAppId: genGetEnvSetting("deviceAppId")
		},
		/* for editors */
		conversions: {
			date: function(name,val,dir) { return (val ? (dir == "set" ? locale.parse : locale.format)(val,{selector:"date",datePattern:DATE_PATTERN}) : (dir == "set" ? null : ""));},
			list: function(name,val,dir) { return (val ? (dir == "set" ? val.toString().split(",") : val.join(",")) : (dir == "set" ? [] : ""));},
			bool: function(name,val,dir) { return (val ? val : (dir == "set" ? null : ""));},
			ssn: function(name,val,dir) { return (val ? (dir == "set" ? (/-/.test(val) ? val : val.substring(0, 3)+"-"+val.substring(3, 5)+"-"+val.substring(5, 9)) : val.toString().replace(/-/g, "")) : ""); },
			phone: function(name,val,dir) { return (val ? (dir == "set" ? (/-/.test(val) ? val : (val.length = 10 ? val.substring(0, 3)+"-"+val.substring(3, 6)+"-"+val.substring(6, 10) : val.substring(0, 3)+"-"+val.substring(3, 7))) : val.toString().replace(/-/g, "")) : ""); },
			zip: function(name,val,dir) { return (val ? (dir == "set" ? (val.length > 5 && !/-/.test(val) ? val.substring(0, 5)+"-"+val.substring(5, 9) : val) : val.toString().replace(/-/g, "")) : ""); },
			number: function(name,val,dir) { return (val ? (dir == "set" ? parseFloat(val) : val) : (val === 0 || val === "0" ? "0" : ""));}
		},
		/* for grids */
		formatters: {
			date: createFormatter("date"),
			tableItem: createFormatter("tableItem"),
			patientPolicy: function(column,store) {
				return createFormatter("tableItem")(column,store,function(item) {
					return item.insurance + (item.policyNum !== "" ? " (Policy #"+item.policyNum+")" : "");
				});
			},
			dollars: createFormatter("dollars"),
			percent: createFormatter("percent"),
			boolean: createFormatter("boolean")
		},
		filters: {
			basic: function(filter,value) {
				if (!filter) {return true;}
				return (filter.length == 0) || filter == value;
			},
			inList: function(filter,value) {
				if (!filter) {return true;}
				return filter.length == 0 || arrayUtils.some(filter,function(item) {return item == value;});
			},
			anyInList: function(filter,list) {
				if (!filter) {return true;}
				return filter.length == 0 || (arrayUtils.indexOf("[NO VALUE]") !== -1 && (!list || list.length === 0)) ||arrayUtils.some(filter,function(item) {
					return arrayUtils.some(list,function(listItem) {
						return item == listItem;
					});
				});
			},
			dateRange: function(filter,value) {
				if (!filter) {return true;}
				var filterStartDate = (filter.start ? new Date(filter.start) : null);
				var filterEndDate = (filter.end ? new Date(filter.end) : null);
				if (!filterStartDate && !filterEndDate) { return true; }
				var parsedValue = locale.parse(value,{selector:"date",datePattern: DATE_PATTERN});
				if (!parsedValue) { return false; }
				return (!filterStartDate || parsedValue.getTime() >= filterStartDate.getTime()) && 
					(!filterEndDate || parsedValue.getTime() <= filterEndDate.getTime());
			}
		},
		validators: {
			policyNum: function(type) {
				return function(name,value) {
					if (value != "") {
						return serverRequest("UI.Utility","ValidatePolicyNum",{
							value: value,
							type: type
						}).then(function(message) {
							if (message !== "") {
								showMessage(message,null,null,"500px");
							}
							return true;
						},function(error) {
							if (error) {
								showMessage(error,null,null,"500px");
							}
							return false;
						});
					}
					return true;
				};
			},
			dateBefore: function(comparisonFieldName,invalidMessage) {
				return function(name,value,lastValues) {
					if (value && lastValues[comparisonFieldName] && timeUtils.floorToDay(lastValues[comparisonFieldName]).getTime() < timeUtils.floorToDay(value).getTime()) {
						if (invalidMessage) {
							showMessage(invalidMessage);
						}
						return false;
					}
					return true;
				};
			},
			dateAfter: function(comparisonFieldName,invalidMessage) {
				return function(name,value,lastValues) {
					if (value && lastValues[comparisonFieldName] && timeUtils.floorToDay(lastValues[comparisonFieldName]).getTime() > timeUtils.floorToDay(value).getTime()) {
						if (invalidMessage) {
							showMessage(invalidMessage);
						}
						return false;
					}
					return true;
				};
			},
			minDate: new Date("12/31/1840")
		},
		patterns:  {
			ssn : {
				value: "\\\\d{3}-\\\\d{2}-\\\\d{4}",
				description: "Please enter SSN in the format ###-##-####"
			},
			phone: {
				value: "(\\\\d{3}-)?\\\\d{3}-\\\\d{4}",
				description: "Please enter number in the format ###-#### or ###-###-####"
			},
			phoneExt: {
				value: "\\\\d*"
			},
			zip: {
				value: "\\\\d{5}(-\\\\d{4})?",
				description: "Please enter ZIP code in the format ##### or #####-####"
			},
			name: {
				value: "[A-Za-z]([A-Za-z]|[ ,.\\'\\\\-])*,[A-Za-z]([A-Za-z]|[,.\\'\\\\-])*([A-Za-z]|[ ,.\\'\\\\-])*",
				description: "Please enter name in the format LASTNAME,FIRSTNAME MI"
			}
		}
	};
});